# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.connection_authentication_purpose import ConnectionAuthenticationPurpose
from ..types.connection_connected_accounts_purpose import ConnectionConnectedAccountsPurpose
from ..types.connection_for_list import ConnectionForList
from ..types.connection_identity_provider_enum import ConnectionIdentityProviderEnum
from ..types.connection_properties_options import ConnectionPropertiesOptions
from ..types.connection_strategy_enum import ConnectionStrategyEnum
from ..types.connections_metadata import ConnectionsMetadata
from ..types.create_connection_response_content import CreateConnectionResponseContent
from ..types.create_cross_app_access_resource_app import CreateCrossAppAccessResourceApp
from ..types.cross_app_access_requesting_app import CrossAppAccessRequestingApp
from ..types.get_connection_response_content import GetConnectionResponseContent
from ..types.list_connections_checkpoint_paginated_response_content import (
    ListConnectionsCheckpointPaginatedResponseContent,
)
from ..types.update_connection_options import UpdateConnectionOptions
from ..types.update_connection_response_content import UpdateConnectionResponseContent
from ..types.update_cross_app_access_resource_app import UpdateCrossAppAccessResourceApp
from .raw_client import AsyncRawConnectionsClient, RawConnectionsClient

if typing.TYPE_CHECKING:
    from .clients.client import AsyncClientsClient, ClientsClient
    from .directory_provisioning.client import AsyncDirectoryProvisioningClient, DirectoryProvisioningClient
    from .keys.client import AsyncKeysClient, KeysClient
    from .scim_configuration.client import AsyncScimConfigurationClient, ScimConfigurationClient
    from .users.client import AsyncUsersClient, UsersClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConnectionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._directory_provisioning: typing.Optional[DirectoryProvisioningClient] = None
        self._scim_configuration: typing.Optional[ScimConfigurationClient] = None
        self._clients: typing.Optional[ClientsClient] = None
        self._keys: typing.Optional[KeysClient] = None
        self._users: typing.Optional[UsersClient] = None

    @property
    def with_raw_response(self) -> RawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConnectionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        name: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]:
        """
        Retrieves detailed list of all [connections](https://auth0.com/docs/authenticate/identity-providers) that match the specified strategy. If no strategy is provided, all connections within your tenant are retrieved. This action can accept a list of fields to include or exclude from the resulting list of connections.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 connections.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        name : typing.Optional[str]
            Provide the name of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]
            The connections were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connections.list(
            from_="from",
            take=1,
            strategy=["ad"],
            name="name",
            fields="fields",
            include_fields=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            from_=from_,
            take=take,
            strategy=strategy,
            name=name,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        strategy: ConnectionIdentityProviderEnum,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[ConnectionPropertiesOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[CreateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectionResponseContent:
        """
        Creates a new connection according to the JSON object received in `body`.

        **Note:** If a connection with the same name was recently deleted and had a large number of associated users, the deletion may still be processing. Creating a new connection with that name before the deletion completes may fail or produce unexpected results.

        Parameters
        ----------
        name : str
            The name of the connection. Must start and end with an alphanumeric character and can only contain alphanumeric characters and '-'. Max length 128

        strategy : ConnectionIdentityProviderEnum

        display_name : typing.Optional[str]
            Connection name used in the new universal login experience

        options : typing.Optional[ConnectionPropertiesOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            Use of this property is NOT RECOMMENDED. Use the PATCH /v2/connections/{id}/clients endpoint to enable the connection for a set of clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[CreateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConnectionResponseContent
            The connection was created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.create(
            name="name",
            strategy="ad",
        )
        """
        _response = self._raw_client.create(
            name=name,
            strategy=strategy,
            display_name=display_name,
            options=options,
            enabled_clients=enabled_clients,
            is_domain_connection=is_domain_connection,
            show_as_button=show_as_button,
            realms=realms,
            metadata=metadata,
            authentication=authentication,
            connected_accounts=connected_accounts,
            cross_app_access_requesting_app=cross_app_access_requesting_app,
            cross_app_access_resource_app=cross_app_access_resource_app,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConnectionResponseContent:
        """
        Retrieve details for a specified [connection](https://auth0.com/docs/authenticate/identity-providers) along with options that can be used for identity provider configuration.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionResponseContent
            The connection was retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.get(
            id="id",
            fields="fields",
            include_fields=True,
        )
        """
        _response = self._raw_client.get(
            id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Removes a specific [connection](https://auth0.com/docs/authenticate/identity-providers) from your tenant. This action cannot be undone. Once removed, users can no longer use this connection to authenticate.

        **Note:** If your connection has a large amount of users associated with it, please be aware that this operation can be long running after the response is returned and may impact concurrent [create connection](https://auth0.com/docs/api/management/v2/connections/post-connections) requests, if they use an identical connection name.

        Parameters
        ----------
        id : str
            The id of the connection to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[UpdateConnectionOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[UpdateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectionResponseContent:
        """
        Update details for a specific [connection](https://auth0.com/docs/authenticate/identity-providers), including option properties for identity provider configuration.

        **Note**: If you use the `options` parameter, the entire `options` object is overridden. To avoid partial data or other issues, ensure all parameters are present when using this option.

        Parameters
        ----------
        id : str
            The id of the connection to update

        display_name : typing.Optional[str]
            The connection name used in the new universal login experience. If display_name is not included in the request, the field will be overwritten with the name value.

        options : typing.Optional[UpdateConnectionOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            DEPRECATED property. Use the PATCH /v2/connections/{id}/clients endpoint to enable or disable the connection for any clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[UpdateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateConnectionResponseContent
            The connection was updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            display_name=display_name,
            options=options,
            enabled_clients=enabled_clients,
            is_domain_connection=is_domain_connection,
            show_as_button=show_as_button,
            realms=realms,
            metadata=metadata,
            authentication=authentication,
            connected_accounts=connected_accounts,
            cross_app_access_requesting_app=cross_app_access_requesting_app,
            cross_app_access_resource_app=cross_app_access_resource_app,
            request_options=request_options,
        )
        return _response.data

    def check_status(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Retrieves the status of an ad/ldap connection referenced by its `ID`. `200 OK` http status code response is returned  when the connection is online, otherwise a `404` status code is returned along with an error message

        Parameters
        ----------
        id : str
            ID of the connection to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.check_status(
            id="id",
        )
        """
        _response = self._raw_client.check_status(id, request_options=request_options)
        return _response.data

    @property
    def directory_provisioning(self):
        if self._directory_provisioning is None:
            from .directory_provisioning.client import DirectoryProvisioningClient  # noqa: E402

            self._directory_provisioning = DirectoryProvisioningClient(client_wrapper=self._client_wrapper)
        return self._directory_provisioning

    @property
    def scim_configuration(self):
        if self._scim_configuration is None:
            from .scim_configuration.client import ScimConfigurationClient  # noqa: E402

            self._scim_configuration = ScimConfigurationClient(client_wrapper=self._client_wrapper)
        return self._scim_configuration

    @property
    def clients(self):
        if self._clients is None:
            from .clients.client import ClientsClient  # noqa: E402

            self._clients = ClientsClient(client_wrapper=self._client_wrapper)
        return self._clients

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import KeysClient  # noqa: E402

            self._keys = KeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def users(self):
        if self._users is None:
            from .users.client import UsersClient  # noqa: E402

            self._users = UsersClient(client_wrapper=self._client_wrapper)
        return self._users


class AsyncConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConnectionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._directory_provisioning: typing.Optional[AsyncDirectoryProvisioningClient] = None
        self._scim_configuration: typing.Optional[AsyncScimConfigurationClient] = None
        self._clients: typing.Optional[AsyncClientsClient] = None
        self._keys: typing.Optional[AsyncKeysClient] = None
        self._users: typing.Optional[AsyncUsersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConnectionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        name: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]:
        """
        Retrieves detailed list of all [connections](https://auth0.com/docs/authenticate/identity-providers) that match the specified strategy. If no strategy is provided, all connections within your tenant are retrieved. This action can accept a list of fields to include or exclude from the resulting list of connections.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 connections.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        name : typing.Optional[str]
            Provide the name of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionForList, ListConnectionsCheckpointPaginatedResponseContent]
            The connections were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connections.list(
                from_="from",
                take=1,
                strategy=["ad"],
                name="name",
                fields="fields",
                include_fields=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            from_=from_,
            take=take,
            strategy=strategy,
            name=name,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        strategy: ConnectionIdentityProviderEnum,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[ConnectionPropertiesOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[CreateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateConnectionResponseContent:
        """
        Creates a new connection according to the JSON object received in `body`.

        **Note:** If a connection with the same name was recently deleted and had a large number of associated users, the deletion may still be processing. Creating a new connection with that name before the deletion completes may fail or produce unexpected results.

        Parameters
        ----------
        name : str
            The name of the connection. Must start and end with an alphanumeric character and can only contain alphanumeric characters and '-'. Max length 128

        strategy : ConnectionIdentityProviderEnum

        display_name : typing.Optional[str]
            Connection name used in the new universal login experience

        options : typing.Optional[ConnectionPropertiesOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            Use of this property is NOT RECOMMENDED. Use the PATCH /v2/connections/{id}/clients endpoint to enable the connection for a set of clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[CreateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConnectionResponseContent
            The connection was created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.create(
                name="name",
                strategy="ad",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            strategy=strategy,
            display_name=display_name,
            options=options,
            enabled_clients=enabled_clients,
            is_domain_connection=is_domain_connection,
            show_as_button=show_as_button,
            realms=realms,
            metadata=metadata,
            authentication=authentication,
            connected_accounts=connected_accounts,
            cross_app_access_requesting_app=cross_app_access_requesting_app,
            cross_app_access_resource_app=cross_app_access_resource_app,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConnectionResponseContent:
        """
        Retrieve details for a specified [connection](https://auth0.com/docs/authenticate/identity-providers) along with options that can be used for identity provider configuration.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConnectionResponseContent
            The connection was retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.get(
                id="id",
                fields="fields",
                include_fields=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Removes a specific [connection](https://auth0.com/docs/authenticate/identity-providers) from your tenant. This action cannot be undone. Once removed, users can no longer use this connection to authenticate.

        **Note:** If your connection has a large amount of users associated with it, please be aware that this operation can be long running after the response is returned and may impact concurrent [create connection](https://auth0.com/docs/api/management/v2/connections/post-connections) requests, if they use an identical connection name.

        Parameters
        ----------
        id : str
            The id of the connection to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        options: typing.Optional[UpdateConnectionOptions] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        is_domain_connection: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        realms: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[ConnectionsMetadata] = OMIT,
        authentication: typing.Optional[ConnectionAuthenticationPurpose] = OMIT,
        connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = OMIT,
        cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = OMIT,
        cross_app_access_resource_app: typing.Optional[UpdateCrossAppAccessResourceApp] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateConnectionResponseContent:
        """
        Update details for a specific [connection](https://auth0.com/docs/authenticate/identity-providers), including option properties for identity provider configuration.

        **Note**: If you use the `options` parameter, the entire `options` object is overridden. To avoid partial data or other issues, ensure all parameters are present when using this option.

        Parameters
        ----------
        id : str
            The id of the connection to update

        display_name : typing.Optional[str]
            The connection name used in the new universal login experience. If display_name is not included in the request, the field will be overwritten with the name value.

        options : typing.Optional[UpdateConnectionOptions]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            DEPRECATED property. Use the PATCH /v2/connections/{id}/clients endpoint to enable or disable the connection for any clients.

        is_domain_connection : typing.Optional[bool]
            <code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)

        show_as_button : typing.Optional[bool]
            Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD. (Defaults to <code>false</code>.)

        realms : typing.Optional[typing.Sequence[str]]
            Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.

        metadata : typing.Optional[ConnectionsMetadata]

        authentication : typing.Optional[ConnectionAuthenticationPurpose]

        connected_accounts : typing.Optional[ConnectionConnectedAccountsPurpose]

        cross_app_access_requesting_app : typing.Optional[CrossAppAccessRequestingApp]

        cross_app_access_resource_app : typing.Optional[UpdateCrossAppAccessResourceApp]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateConnectionResponseContent
            The connection was updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            display_name=display_name,
            options=options,
            enabled_clients=enabled_clients,
            is_domain_connection=is_domain_connection,
            show_as_button=show_as_button,
            realms=realms,
            metadata=metadata,
            authentication=authentication,
            connected_accounts=connected_accounts,
            cross_app_access_requesting_app=cross_app_access_requesting_app,
            cross_app_access_resource_app=cross_app_access_resource_app,
            request_options=request_options,
        )
        return _response.data

    async def check_status(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Retrieves the status of an ad/ldap connection referenced by its `ID`. `200 OK` http status code response is returned  when the connection is online, otherwise a `404` status code is returned along with an error message

        Parameters
        ----------
        id : str
            ID of the connection to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.check_status(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.check_status(id, request_options=request_options)
        return _response.data

    @property
    def directory_provisioning(self):
        if self._directory_provisioning is None:
            from .directory_provisioning.client import AsyncDirectoryProvisioningClient  # noqa: E402

            self._directory_provisioning = AsyncDirectoryProvisioningClient(client_wrapper=self._client_wrapper)
        return self._directory_provisioning

    @property
    def scim_configuration(self):
        if self._scim_configuration is None:
            from .scim_configuration.client import AsyncScimConfigurationClient  # noqa: E402

            self._scim_configuration = AsyncScimConfigurationClient(client_wrapper=self._client_wrapper)
        return self._scim_configuration

    @property
    def clients(self):
        if self._clients is None:
            from .clients.client import AsyncClientsClient  # noqa: E402

            self._clients = AsyncClientsClient(client_wrapper=self._client_wrapper)
        return self._clients

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import AsyncKeysClient  # noqa: E402

            self._keys = AsyncKeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def users(self):
        if self._users is None:
            from .users.client import AsyncUsersClient  # noqa: E402

            self._users = AsyncUsersClient(client_wrapper=self._client_wrapper)
        return self._users



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionShouldTrustEmailVerifiedConnectionEnum = typing.Union[
    typing.Literal["never_set_emails_as_verified", "always_set_emails_as_verified"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_customDomains_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "custom_domains.list_.0"
    client = get_client(test_id)
    client.custom_domains.list(
        q="q",
        fields="fields",
        include_fields=True,
        sort="sort",
    )
    verify_request_count(
        test_id, "GET", "/custom-domains", {"q": "q", "fields": "fields", "include_fields": "true", "sort": "sort"}, 1
    )


def test_customDomains_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "custom_domains.create.0"
    client = get_client(test_id)
    client.custom_domains.create(
        domain="domain",
        type="auth0_managed_certs",
    )
    verify_request_count(test_id, "POST", "/custom-domains", None, 1)


def test_customDomains_get_default() -> None:
    """Test getDefault endpoint with WireMock"""
    test_id = "custom_domains.get_default.0"
    client = get_client(test_id)
    client.custom_domains.get_default()
    verify_request_count(test_id, "GET", "/custom-domains/default", None, 1)


def test_customDomains_set_default() -> None:
    """Test setDefault endpoint with WireMock"""
    test_id = "custom_domains.set_default.0"
    client = get_client(test_id)
    client.custom_domains.set_default(
        domain="domain",
    )
    verify_request_count(test_id, "PATCH", "/custom-domains/default", None, 1)


def test_customDomains_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "custom_domains.get.0"
    client = get_client(test_id)
    client.custom_domains.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/custom-domains/id", None, 1)


def test_customDomains_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "custom_domains.delete.0"
    client = get_client(test_id)
    client.custom_domains.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/custom-domains/id", None, 1)


def test_customDomains_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "custom_domains.update.0"
    client = get_client(test_id)
    client.custom_domains.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/custom-domains/id", None, 1)


def test_customDomains_test() -> None:
    """Test test endpoint with WireMock"""
    test_id = "custom_domains.test.0"
    client = get_client(test_id)
    client.custom_domains.test(
        id="id",
    )
    verify_request_count(test_id, "POST", "/custom-domains/id/test", None, 1)


def test_customDomains_verify() -> None:
    """Test verify endpoint with WireMock"""
    test_id = "custom_domains.verify.0"
    client = get_client(test_id)
    client.custom_domains.verify(
        id="id",
    )
    verify_request_count(test_id, "POST", "/custom-domains/id/verify", None, 1)



from .conftest import get_client, verify_request_count


def test_groups_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "groups.list_.0"
    client = get_client(test_id)
    client.groups.list(
        connection_id="connection_id",
        name="name",
        external_id="external_id",
        search="search",
        fields="fields",
        include_fields=True,
        from_="from",
        take=1,
    )
    verify_request_count(
        test_id,
        "GET",
        "/groups",
        {
            "connection_id": "connection_id",
            "name": "name",
            "external_id": "external_id",
            "search": "search",
            "fields": "fields",
            "include_fields": "true",
            "from": "from",
            "take": "1",
        },
        1,
    )


def test_groups_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "groups.get.0"
    client = get_client(test_id)
    client.groups.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/groups/id", None, 1)


def test_groups_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "groups.delete.0"
    client = get_client(test_id)
    client.groups.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/groups/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_activecampaign import CreateFlowsVaultConnectionActivecampaign
from .create_flows_vault_connection_airtable import CreateFlowsVaultConnectionAirtable
from .create_flows_vault_connection_auth_0 import CreateFlowsVaultConnectionAuth0
from .create_flows_vault_connection_bigquery import CreateFlowsVaultConnectionBigquery
from .create_flows_vault_connection_clearbit import CreateFlowsVaultConnectionClearbit
from .create_flows_vault_connection_docusign import CreateFlowsVaultConnectionDocusign
from .create_flows_vault_connection_google_sheets import CreateFlowsVaultConnectionGoogleSheets
from .create_flows_vault_connection_http import CreateFlowsVaultConnectionHttp
from .create_flows_vault_connection_hubspot import CreateFlowsVaultConnectionHubspot
from .create_flows_vault_connection_jwt import CreateFlowsVaultConnectionJwt
from .create_flows_vault_connection_mailchimp import CreateFlowsVaultConnectionMailchimp
from .create_flows_vault_connection_mailjet import CreateFlowsVaultConnectionMailjet
from .create_flows_vault_connection_pipedrive import CreateFlowsVaultConnectionPipedrive
from .create_flows_vault_connection_salesforce import CreateFlowsVaultConnectionSalesforce
from .create_flows_vault_connection_sendgrid import CreateFlowsVaultConnectionSendgrid
from .create_flows_vault_connection_slack import CreateFlowsVaultConnectionSlack
from .create_flows_vault_connection_stripe import CreateFlowsVaultConnectionStripe
from .create_flows_vault_connection_telegram import CreateFlowsVaultConnectionTelegram
from .create_flows_vault_connection_twilio import CreateFlowsVaultConnectionTwilio
from .create_flows_vault_connection_whatsapp import CreateFlowsVaultConnectionWhatsapp
from .create_flows_vault_connection_zapier import CreateFlowsVaultConnectionZapier

CreateFlowsVaultConnectionRequestContent = typing.Union[
    CreateFlowsVaultConnectionActivecampaign,
    CreateFlowsVaultConnectionAirtable,
    CreateFlowsVaultConnectionAuth0,
    CreateFlowsVaultConnectionBigquery,
    CreateFlowsVaultConnectionClearbit,
    CreateFlowsVaultConnectionDocusign,
    CreateFlowsVaultConnectionGoogleSheets,
    CreateFlowsVaultConnectionHttp,
    CreateFlowsVaultConnectionHubspot,
    CreateFlowsVaultConnectionJwt,
    CreateFlowsVaultConnectionMailchimp,
    CreateFlowsVaultConnectionMailjet,
    CreateFlowsVaultConnectionPipedrive,
    CreateFlowsVaultConnectionSalesforce,
    CreateFlowsVaultConnectionSendgrid,
    CreateFlowsVaultConnectionSlack,
    CreateFlowsVaultConnectionStripe,
    CreateFlowsVaultConnectionTelegram,
    CreateFlowsVaultConnectionTwilio,
    CreateFlowsVaultConnectionWhatsapp,
    CreateFlowsVaultConnectionZapier,
]



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamSubscribeEventsEventTypeEnum = typing.Union[
    typing.Literal[
        "connection.created",
        "connection.deleted",
        "connection.updated",
        "group.created",
        "group.deleted",
        "group.member.added",
        "group.member.deleted",
        "group.role.assigned",
        "group.role.deleted",
        "group.updated",
        "organization.connection.added",
        "organization.connection.removed",
        "organization.connection.updated",
        "organization.created",
        "organization.deleted",
        "organization.group.role.assigned",
        "organization.group.role.deleted",
        "organization.member.added",
        "organization.member.deleted",
        "organization.member.role.assigned",
        "organization.member.role.deleted",
        "organization.updated",
        "user.created",
        "user.deleted",
        "user.updated",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

ConnectionDebugSaml = bool
"""
When true, enables detailed SAML debugging by issuing 'w' (warning) events in tenant logs containing SAML request/response details. WARNING: Potentially exposes sensitive user information (PII, credentials) and should only be enabled temporarily for debugging purposes.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionTokenEndpointAuthMethodsSupported = typing.List[str]
"""
JSON array containing a list of Client Authentication methods supported by this Token Endpoint. The options are client_secret_post, client_secret_basic, client_secret_jwt, and private_key_jwt, as described in Section 9 of OpenID Connect Core 1.0 [OpenID.Core]. Other authentication methods MAY be defined by extensions. If omitted, the default is client_secret_basic -- the HTTP Basic Authentication Scheme specified in Section 2.3.1 of OAuth 2.0 [RFC6749].
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .encryption_key_state import EncryptionKeyState
from .encryption_key_type import EncryptionKeyType


class EncryptionKey(UniversalBaseModel):
    """
    Encryption key
    """

    kid: str = pydantic.Field()
    """
    Key ID
    """

    type: EncryptionKeyType
    state: EncryptionKeyState
    created_at: dt.datetime = pydantic.Field()
    """
    Key creation timestamp
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Key update timestamp
    """

    parent_kid: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of parent wrapping key
    """

    public_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Public key in PEM format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_branding_response_content import GetBrandingResponseContent
from ..types.update_branding_colors import UpdateBrandingColors
from ..types.update_branding_font import UpdateBrandingFont
from ..types.update_branding_response_content import UpdateBrandingResponseContent
from .raw_client import AsyncRawBrandingClient, RawBrandingClient

if typing.TYPE_CHECKING:
    from .phone.client import AsyncPhoneClient, PhoneClient
    from .templates.client import AsyncTemplatesClient, TemplatesClient
    from .themes.client import AsyncThemesClient, ThemesClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BrandingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBrandingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._templates: typing.Optional[TemplatesClient] = None
        self._themes: typing.Optional[ThemesClient] = None
        self._phone: typing.Optional[PhoneClient] = None

    @property
    def with_raw_response(self) -> RawBrandingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBrandingClient
        """
        return self._raw_client

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetBrandingResponseContent:
        """
        Retrieve branding settings.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingResponseContent
            Branding settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        colors: typing.Optional[UpdateBrandingColors] = OMIT,
        favicon_url: typing.Optional[str] = OMIT,
        logo_url: typing.Optional[str] = OMIT,
        font: typing.Optional[UpdateBrandingFont] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBrandingResponseContent:
        """
        Update branding settings.

        Parameters
        ----------
        colors : typing.Optional[UpdateBrandingColors]

        favicon_url : typing.Optional[str]
            URL for the favicon. Must use HTTPS.

        logo_url : typing.Optional[str]
            URL for the logo. Must use HTTPS.

        font : typing.Optional[UpdateBrandingFont]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBrandingResponseContent
            Branding settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.update()
        """
        _response = self._raw_client.update(
            colors=colors, favicon_url=favicon_url, logo_url=logo_url, font=font, request_options=request_options
        )
        return _response.data

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import TemplatesClient  # noqa: E402

            self._templates = TemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates

    @property
    def themes(self):
        if self._themes is None:
            from .themes.client import ThemesClient  # noqa: E402

            self._themes = ThemesClient(client_wrapper=self._client_wrapper)
        return self._themes

    @property
    def phone(self):
        if self._phone is None:
            from .phone.client import PhoneClient  # noqa: E402

            self._phone = PhoneClient(client_wrapper=self._client_wrapper)
        return self._phone


class AsyncBrandingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBrandingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._templates: typing.Optional[AsyncTemplatesClient] = None
        self._themes: typing.Optional[AsyncThemesClient] = None
        self._phone: typing.Optional[AsyncPhoneClient] = None

    @property
    def with_raw_response(self) -> AsyncRawBrandingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBrandingClient
        """
        return self._raw_client

    async def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetBrandingResponseContent:
        """
        Retrieve branding settings.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingResponseContent
            Branding settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        colors: typing.Optional[UpdateBrandingColors] = OMIT,
        favicon_url: typing.Optional[str] = OMIT,
        logo_url: typing.Optional[str] = OMIT,
        font: typing.Optional[UpdateBrandingFont] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBrandingResponseContent:
        """
        Update branding settings.

        Parameters
        ----------
        colors : typing.Optional[UpdateBrandingColors]

        favicon_url : typing.Optional[str]
            URL for the favicon. Must use HTTPS.

        logo_url : typing.Optional[str]
            URL for the logo. Must use HTTPS.

        font : typing.Optional[UpdateBrandingFont]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBrandingResponseContent
            Branding settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            colors=colors, favicon_url=favicon_url, logo_url=logo_url, font=font, request_options=request_options
        )
        return _response.data

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import AsyncTemplatesClient  # noqa: E402

            self._templates = AsyncTemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates

    @property
    def themes(self):
        if self._themes is None:
            from .themes.client import AsyncThemesClient  # noqa: E402

            self._themes = AsyncThemesClient(client_wrapper=self._client_wrapper)
        return self._themes

    @property
    def phone(self):
        if self._phone is None:
            from .phone.client import AsyncPhoneClient  # noqa: E402

            self._phone = AsyncPhoneClient(client_wrapper=self._client_wrapper)
        return self._phone



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionDomainAliasesAd = typing.List[str]
"""
List of domain names that can be used with identifier-first authentication flow to route users to this AD connection; each domain must be a valid DNS name up to 256 characters
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.create_scim_token_response_content import CreateScimTokenResponseContent
from ....types.get_scim_tokens_response_content import GetScimTokensResponseContent
from .raw_client import AsyncRawTokensClient, RawTokensClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TokensClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTokensClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTokensClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTokensClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetScimTokensResponseContent:
        """
        Retrieves all scim tokens by its connection `id`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimTokensResponseContent
            The connection's SCIM tokens were retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.tokens.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        scopes: typing.Optional[typing.Sequence[str]] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateScimTokenResponseContent:
        """
        Create a scim token for a scim client.

        Parameters
        ----------
        id : str
            The id of the connection to create its SCIM token

        scopes : typing.Optional[typing.Sequence[str]]
            The scopes of the scim token

        token_lifetime : typing.Optional[int]
            Lifetime of the token in seconds. Must be greater than 900

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateScimTokenResponseContent
            The connection's SCIM token was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.tokens.create(
            id="id",
        )
        """
        _response = self._raw_client.create(
            id, scopes=scopes, token_lifetime=token_lifetime, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, token_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a scim token by its connection `id` and `tokenId`.

        Parameters
        ----------
        id : str
            The connection id that owns the SCIM token to delete

        token_id : str
            The id of the scim token to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.tokens.delete(
            id="id",
            token_id="tokenId",
        )
        """
        _response = self._raw_client.delete(id, token_id, request_options=request_options)
        return _response.data


class AsyncTokensClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTokensClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTokensClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTokensClient
        """
        return self._raw_client

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScimTokensResponseContent:
        """
        Retrieves all scim tokens by its connection `id`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimTokensResponseContent
            The connection's SCIM tokens were retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.tokens.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        scopes: typing.Optional[typing.Sequence[str]] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateScimTokenResponseContent:
        """
        Create a scim token for a scim client.

        Parameters
        ----------
        id : str
            The id of the connection to create its SCIM token

        scopes : typing.Optional[typing.Sequence[str]]
            The scopes of the scim token

        token_lifetime : typing.Optional[int]
            Lifetime of the token in seconds. Must be greater than 900

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateScimTokenResponseContent
            The connection's SCIM token was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.tokens.create(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            id, scopes=scopes, token_lifetime=token_lifetime, request_options=request_options
        )
        return _response.data

    async def delete(self, id: str, token_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a scim token by its connection `id` and `tokenId`.

        Parameters
        ----------
        id : str
            The connection id that owns the SCIM token to delete

        token_id : str
            The id of the scim token to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.tokens.delete(
                id="id",
                token_id="tokenId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, token_id, request_options=request_options)
        return _response.data



from .conftest import get_client, verify_request_count


def test_organizations_discoveryDomains_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.discovery_domains.list_.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.list(
        id="id",
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/organizations/id/discovery-domains", {"from": "from", "take": "1"}, 1)


def test_organizations_discoveryDomains_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "organizations.discovery_domains.create.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.create(
        id="id",
        domain="domain",
    )
    verify_request_count(test_id, "POST", "/organizations/id/discovery-domains", None, 1)


def test_organizations_discoveryDomains_get_by_name() -> None:
    """Test getByName endpoint with WireMock"""
    test_id = "organizations.discovery_domains.get_by_name.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.get_by_name(
        id="id",
        discovery_domain="discovery_domain",
    )
    verify_request_count(test_id, "GET", "/organizations/id/discovery-domains/name/discovery_domain", None, 1)


def test_organizations_discoveryDomains_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "organizations.discovery_domains.get.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.get(
        id="id",
        discovery_domain_id="discovery_domain_id",
    )
    verify_request_count(test_id, "GET", "/organizations/id/discovery-domains/discovery_domain_id", None, 1)


def test_organizations_discoveryDomains_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "organizations.discovery_domains.delete.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.delete(
        id="id",
        discovery_domain_id="discovery_domain_id",
    )
    verify_request_count(test_id, "DELETE", "/organizations/id/discovery-domains/discovery_domain_id", None, 1)


def test_organizations_discoveryDomains_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "organizations.discovery_domains.update.0"
    client = get_client(test_id)
    client.organizations.discovery_domains.update(
        id="id",
        discovery_domain_id="discovery_domain_id",
    )
    verify_request_count(test_id, "PATCH", "/organizations/id/discovery-domains/discovery_domain_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .self_service_profile_sso_ticket_idp_initiated_client_protocol_enum import (
    SelfServiceProfileSsoTicketIdpInitiatedClientProtocolEnum,
)


class SelfServiceProfileSsoTicketIdpInitiatedOptions(UniversalBaseModel):
    """
    Allows IdP-initiated login
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables IdP-initiated login for this connection
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default application <code>client_id</code> user is redirected to after validated SAML response
    """

    client_protocol: typing.Optional[SelfServiceProfileSsoTicketIdpInitiatedClientProtocolEnum] = None
    client_authorizequery: typing.Optional[str] = pydantic.Field(default=None)
    """
    Query string options to customize the behaviour for OpenID Connect when <code>idpinitiated.client_protocol</code> is <code>oauth2</code>. Allowed parameters: <code>redirect_uri</code>, <code>scope</code>, <code>response_type</code>. For example, <code>redirect_uri=https://jwt.io&scope=openid email&response_type=token</code>
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_flow_map_value_params_cases import FlowActionFlowMapValueParamsCases
from .flow_action_flow_map_value_params_fallback import FlowActionFlowMapValueParamsFallback
from .flow_action_flow_map_value_params_input import FlowActionFlowMapValueParamsInput


class FlowActionFlowMapValueParams(UniversalBaseModel):
    input: FlowActionFlowMapValueParamsInput
    cases: typing.Optional[FlowActionFlowMapValueParamsCases] = None
    fallback: typing.Optional[FlowActionFlowMapValueParamsFallback] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_connections_scimConfiguration_tokens_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "connections.scim_configuration.tokens.get.0"
    client = get_client(test_id)
    client.connections.scim_configuration.tokens.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connections/id/scim-configuration/tokens", None, 1)


def test_connections_scimConfiguration_tokens_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "connections.scim_configuration.tokens.create.0"
    client = get_client(test_id)
    client.connections.scim_configuration.tokens.create(
        id="id",
    )
    verify_request_count(test_id, "POST", "/connections/id/scim-configuration/tokens", None, 1)


def test_connections_scimConfiguration_tokens_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "connections.scim_configuration.tokens.delete.0"
    client = get_client(test_id)
    client.connections.scim_configuration.tokens.delete(
        id="id",
        token_id="tokenId",
    )
    verify_request_count(test_id, "DELETE", "/connections/id/scim-configuration/tokens/tokenId", None, 1)



from .conftest import get_client, verify_request_count


def test_clients_credentials_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "clients.credentials.list_.0"
    client = get_client(test_id)
    client.clients.credentials.list(
        client_id="client_id",
    )
    verify_request_count(test_id, "GET", "/clients/client_id/credentials", None, 1)


def test_clients_credentials_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "clients.credentials.create.0"
    client = get_client(test_id)
    client.clients.credentials.create(
        client_id="client_id",
        credential_type="public_key",
    )
    verify_request_count(test_id, "POST", "/clients/client_id/credentials", None, 1)


def test_clients_credentials_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "clients.credentials.get.0"
    client = get_client(test_id)
    client.clients.credentials.get(
        client_id="client_id",
        credential_id="credential_id",
    )
    verify_request_count(test_id, "GET", "/clients/client_id/credentials/credential_id", None, 1)


def test_clients_credentials_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "clients.credentials.delete.0"
    client = get_client(test_id)
    client.clients.credentials.delete(
        client_id="client_id",
        credential_id="credential_id",
    )
    verify_request_count(test_id, "DELETE", "/clients/client_id/credentials/credential_id", None, 1)


def test_clients_credentials_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "clients.credentials.update.0"
    client = get_client(test_id)
    client.clients.credentials.update(
        client_id="client_id",
        credential_id="credential_id",
    )
    verify_request_count(test_id, "PATCH", "/clients/client_id/credentials/credential_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_subscribe_events_event_type_enum import EventStreamSubscribeEventsEventTypeEnum

EventStreamSubscribeEventsEventTypeParam = typing.List[EventStreamSubscribeEventsEventTypeEnum]
"""
Event type(s) to listen for. Specify multiple times for multiple types (e.g., ?event_type=user.created&event_type=user.updated). If not provided, all event types will be streamed.
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.list_guardian_policies_response_content import ListGuardianPoliciesResponseContent
from ...types.set_guardian_policies_request_content import SetGuardianPoliciesRequestContent
from ...types.set_guardian_policies_response_content import SetGuardianPoliciesResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPoliciesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListGuardianPoliciesResponseContent]:
        """
        Retrieve the [multi-factor authentication (MFA) policies](https://auth0.com/docs/secure/multi-factor-authentication/enable-mfa) configured for your tenant.

        The following policies are supported:

        - `all-applications` policy prompts with MFA for all logins.
        - `confidence-score` policy prompts with MFA only for low confidence logins.

        **Note**: The `confidence-score` policy is part of the [Adaptive MFA feature](https://auth0.com/docs/secure/multi-factor-authentication/adaptive-mfa). Adaptive MFA requires an add-on for the Enterprise plan; review [Auth0 Pricing](https://auth0.com/pricing) for more details.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListGuardianPoliciesResponseContent]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/policies",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListGuardianPoliciesResponseContent,
                    parse_obj_as(
                        type_=ListGuardianPoliciesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set(
        self, *, request: SetGuardianPoliciesRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SetGuardianPoliciesResponseContent]:
        """
        Set [multi-factor authentication (MFA) policies](https://auth0.com/docs/secure/multi-factor-authentication/enable-mfa) for your tenant.

        The following policies are supported:

        - `all-applications` policy prompts with MFA for all logins.
        - `confidence-score` policy prompts with MFA only for low confidence logins.

        **Note**: The `confidence-score` policy is part of the [Adaptive MFA feature](https://auth0.com/docs/secure/multi-factor-authentication/adaptive-mfa). Adaptive MFA requires an add-on for the Enterprise plan; review [Auth0 Pricing](https://auth0.com/pricing) for more details.

        Parameters
        ----------
        request : SetGuardianPoliciesRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianPoliciesResponseContent]
            Policies updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/policies",
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianPoliciesResponseContent,
                    parse_obj_as(
                        type_=SetGuardianPoliciesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPoliciesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListGuardianPoliciesResponseContent]:
        """
        Retrieve the [multi-factor authentication (MFA) policies](https://auth0.com/docs/secure/multi-factor-authentication/enable-mfa) configured for your tenant.

        The following policies are supported:

        - `all-applications` policy prompts with MFA for all logins.
        - `confidence-score` policy prompts with MFA only for low confidence logins.

        **Note**: The `confidence-score` policy is part of the [Adaptive MFA feature](https://auth0.com/docs/secure/multi-factor-authentication/adaptive-mfa). Adaptive MFA requires an add-on for the Enterprise plan; review [Auth0 Pricing](https://auth0.com/pricing) for more details.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListGuardianPoliciesResponseContent]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/policies",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListGuardianPoliciesResponseContent,
                    parse_obj_as(
                        type_=ListGuardianPoliciesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set(
        self, *, request: SetGuardianPoliciesRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SetGuardianPoliciesResponseContent]:
        """
        Set [multi-factor authentication (MFA) policies](https://auth0.com/docs/secure/multi-factor-authentication/enable-mfa) for your tenant.

        The following policies are supported:

        - `all-applications` policy prompts with MFA for all logins.
        - `confidence-score` policy prompts with MFA only for low confidence logins.

        **Note**: The `confidence-score` policy is part of the [Adaptive MFA feature](https://auth0.com/docs/secure/multi-factor-authentication/adaptive-mfa). Adaptive MFA requires an add-on for the Enterprise plan; review [Auth0 Pricing](https://auth0.com/pricing) for more details.

        Parameters
        ----------
        request : SetGuardianPoliciesRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianPoliciesResponseContent]
            Policies updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/policies",
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianPoliciesResponseContent,
                    parse_obj_as(
                        type_=SetGuardianPoliciesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionUserinfoSigningAlgValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWS [JWS] signing algorithms (alg values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT]. The value none MAY be included.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_2048 import ConnectionHttpsUrlWithHttpFallback2048

ConnectionSignInEndpointSaml = ConnectionHttpsUrlWithHttpFallback2048
"""
Identity provider's SAML SingleSignOnService endpoint URL where Auth0 sends SAML authentication requests. This is the primary login URL for the SAML connection. Required unless using metadataUrl or metadataXml.
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_log_stream_request_content import CreateLogStreamRequestContent
from ..types.create_log_stream_response_content import CreateLogStreamResponseContent
from ..types.get_log_stream_response_content import GetLogStreamResponseContent
from ..types.log_stream_filter import LogStreamFilter
from ..types.log_stream_pii_config import LogStreamPiiConfig
from ..types.log_stream_response_schema import LogStreamResponseSchema
from ..types.log_stream_sink_patch import LogStreamSinkPatch
from ..types.log_stream_status_enum import LogStreamStatusEnum
from ..types.update_log_stream_response_content import UpdateLogStreamResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLogStreamsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[LogStreamResponseSchema]]:
        """
        Retrieve details on [log streams](https://auth0.com/docs/logs/streams).

        **Sample Response**

        ```json
        [{
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }, {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "eventgrid",
          "status": "active|paused|suspended",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }]
        ```

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[LogStreamResponseSchema]]
            Returning log streams
        """
        _response = self._client_wrapper.httpx_client.request(
            "log-streams",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[LogStreamResponseSchema],
                    parse_obj_as(
                        type_=typing.List[LogStreamResponseSchema],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: CreateLogStreamRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateLogStreamResponseContent]:
        """
        Create a log stream.

        **Log Stream Types**

        The `type` of log stream being created determines the properties required in the `sink` payload.

        **HTTP Stream**

        For an `http` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "http",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Amazon EventBridge Stream**

        For an `eventbridge` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventbridge",
          "sink": {
            "awsRegion": "string",
            "awsAccountId": "string"
          }
        }
        ```

        The response will include an additional field `awsPartnerEventSource` in the `sink`:

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **Azure Event Grid Stream**

        For an `Azure Event Grid` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventgrid",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        }
        ```

        **Datadog Stream**

        For a `Datadog` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "datadog",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Splunk Stream**

        For a `Splunk` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "splunk",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Stream**

        For a `Sumo Logic` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "sumo",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        request : CreateLogStreamRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateLogStreamResponseContent]
            Log stream created
        """
        _response = self._client_wrapper.httpx_client.request(
            "log-streams",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateLogStreamRequestContent, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLogStreamResponseContent,
                    parse_obj_as(
                        type_=CreateLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetLogStreamResponseContent]:
        """
        Retrieve a log stream configuration and status.

        **Sample responses**

        **Amazon EventBridge Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **HTTP Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        }
        ```

        **Datadog Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Mixpanel**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "mixpanel",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "mixpanel",
          "status": "active",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Segment**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "segment",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "segment",
          "status": "active",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Splunk Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Status**

        The `status` of a log stream maybe any of the following:

        1. `active` - Stream is currently enabled.
        2. `paused` - Stream is currently user disabled and will not attempt log delivery.
        3. `suspended` - Stream is currently disabled because of errors and will not attempt log delivery.

        Parameters
        ----------
        id : str
            The id of the log stream to get

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLogStreamResponseContent]
            Returning log stream.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogStreamResponseContent,
                    parse_obj_as(
                        type_=GetLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a log stream.

        Parameters
        ----------
        id : str
            The id of the log stream to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[LogStreamStatusEnum] = OMIT,
        is_priority: typing.Optional[bool] = OMIT,
        filters: typing.Optional[typing.Sequence[LogStreamFilter]] = OMIT,
        pii_config: typing.Optional[LogStreamPiiConfig] = OMIT,
        sink: typing.Optional[LogStreamSinkPatch] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateLogStreamResponseContent]:
        """
        Update a log stream.

        **Examples of how to use the PATCH endpoint.**

        The following fields may be updated in a PATCH operation:

        - name
        - status
        - sink

        Note: For log streams of type `eventbridge` and `eventgrid`, updating the `sink` is not permitted.

        **Update the status of a log stream**

        ```json
        {
          "status": "active|paused"
        }
        ```

        **Update the name of a log stream**

        ```json
        {
          "name": "string"
        }
        ```

        **Update the sink properties of a stream of type `http`**

        ```json
        {
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONARRAY|JSONLINES",
            "httpAuthorization": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `datadog`**

        ```json
        {
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `splunk`**

        ```json
        {
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Update the sink properties of a stream of type `sumo`**

        ```json
        {
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        id : str
            The id of the log stream to get

        name : typing.Optional[str]
            log stream name

        status : typing.Optional[LogStreamStatusEnum]

        is_priority : typing.Optional[bool]
            True for priority log streams, false for non-priority

        filters : typing.Optional[typing.Sequence[LogStreamFilter]]
            Only logs events matching these filters will be delivered by the stream. If omitted or empty, all events will be delivered.

        pii_config : typing.Optional[LogStreamPiiConfig]

        sink : typing.Optional[LogStreamSinkPatch]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateLogStreamResponseContent]
            Log stream updated
        """
        _response = self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "status": status,
                "isPriority": is_priority,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Sequence[LogStreamFilter], direction="write"
                ),
                "pii_config": convert_and_respect_annotation_metadata(
                    object_=pii_config, annotation=LogStreamPiiConfig, direction="write"
                ),
                "sink": convert_and_respect_annotation_metadata(
                    object_=sink, annotation=LogStreamSinkPatch, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLogStreamResponseContent,
                    parse_obj_as(
                        type_=UpdateLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLogStreamsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[LogStreamResponseSchema]]:
        """
        Retrieve details on [log streams](https://auth0.com/docs/logs/streams).

        **Sample Response**

        ```json
        [{
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }, {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "eventgrid",
          "status": "active|paused|suspended",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        },
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }]
        ```

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[LogStreamResponseSchema]]
            Returning log streams
        """
        _response = await self._client_wrapper.httpx_client.request(
            "log-streams",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[LogStreamResponseSchema],
                    parse_obj_as(
                        type_=typing.List[LogStreamResponseSchema],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: CreateLogStreamRequestContent, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateLogStreamResponseContent]:
        """
        Create a log stream.

        **Log Stream Types**

        The `type` of log stream being created determines the properties required in the `sink` payload.

        **HTTP Stream**

        For an `http` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "http",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpAuthorization": "string"
          }
        }
        ```

        **Amazon EventBridge Stream**

        For an `eventbridge` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventbridge",
          "sink": {
            "awsRegion": "string",
            "awsAccountId": "string"
          }
        }
        ```

        The response will include an additional field `awsPartnerEventSource` in the `sink`:

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **Azure Event Grid Stream**

        For an `Azure Event Grid` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "eventgrid",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active",
          "sink": {
            "azureSubscriptionId": "string",
            "azureResourceGroup": "string",
            "azureRegion": "string",
            "azurePartnerTopic": "string"
          }
        }
        ```

        **Datadog Stream**

        For a `Datadog` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "datadog",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Splunk Stream**

        For a `Splunk` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "splunk",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Stream**

        For a `Sumo Logic` Stream, the `sink` properties are listed in the payload below.

        **Request:**
        ```json
        {
          "name": "string",
          "type": "sumo",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Response:**
        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        request : CreateLogStreamRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateLogStreamResponseContent]
            Log stream created
        """
        _response = await self._client_wrapper.httpx_client.request(
            "log-streams",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateLogStreamRequestContent, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLogStreamResponseContent,
                    parse_obj_as(
                        type_=CreateLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetLogStreamResponseContent]:
        """
        Retrieve a log stream configuration and status.

        **Sample responses**

        **Amazon EventBridge Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "eventbridge",
          "status": "active|paused|suspended",
          "sink": {
            "awsAccountId": "string",
            "awsRegion": "string",
            "awsPartnerEventSource": "string"
          }
        }
        ```

        **HTTP Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "http",
          "status": "active|paused|suspended",
          "sink": {
            "httpContentFormat": "JSONLINES|JSONARRAY",
            "httpContentType": "string",
            "httpEndpoint": "string",
            "httpAuthorization": "string"
          }
        }
        ```

        **Datadog Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "datadog",
          "status": "active|paused|suspended",
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Mixpanel**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "mixpanel",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "mixpanel",
          "status": "active",
          "sink": {
            "mixpanelRegion": "string",
            "mixpanelProjectId": "string",
            "mixpanelServiceAccountUsername": "string",
            "mixpanelServiceAccountPassword": "string"
          }
        }
        ```

        **Segment**

        **Request:**

        ```json
        {
          "name": "string",
          "type": "segment",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Response:**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "segment",
          "status": "active",
          "sink": {
            "segmentWriteKey": "string"
          }
        }
        ```

        **Splunk Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "splunk",
          "status": "active|paused|suspended",
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Sumo Logic Log Stream**

        ```json
        {
          "id": "string",
          "name": "string",
          "type": "sumo",
          "status": "active|paused|suspended",
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        **Status**

        The `status` of a log stream maybe any of the following:

        1. `active` - Stream is currently enabled.
        2. `paused` - Stream is currently user disabled and will not attempt log delivery.
        3. `suspended` - Stream is currently disabled because of errors and will not attempt log delivery.

        Parameters
        ----------
        id : str
            The id of the log stream to get

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLogStreamResponseContent]
            Returning log stream.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogStreamResponseContent,
                    parse_obj_as(
                        type_=GetLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a log stream.

        Parameters
        ----------
        id : str
            The id of the log stream to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[LogStreamStatusEnum] = OMIT,
        is_priority: typing.Optional[bool] = OMIT,
        filters: typing.Optional[typing.Sequence[LogStreamFilter]] = OMIT,
        pii_config: typing.Optional[LogStreamPiiConfig] = OMIT,
        sink: typing.Optional[LogStreamSinkPatch] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateLogStreamResponseContent]:
        """
        Update a log stream.

        **Examples of how to use the PATCH endpoint.**

        The following fields may be updated in a PATCH operation:

        - name
        - status
        - sink

        Note: For log streams of type `eventbridge` and `eventgrid`, updating the `sink` is not permitted.

        **Update the status of a log stream**

        ```json
        {
          "status": "active|paused"
        }
        ```

        **Update the name of a log stream**

        ```json
        {
          "name": "string"
        }
        ```

        **Update the sink properties of a stream of type `http`**

        ```json
        {
          "sink": {
            "httpEndpoint": "string",
            "httpContentType": "string",
            "httpContentFormat": "JSONARRAY|JSONLINES",
            "httpAuthorization": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `datadog`**

        ```json
        {
          "sink": {
            "datadogRegion": "string",
            "datadogApiKey": "string"
          }
        }
        ```

        **Update the sink properties of a stream of type `splunk`**

        ```json
        {
          "sink": {
            "splunkDomain": "string",
            "splunkToken": "string",
            "splunkPort": "string",
            "splunkSecure": "boolean"
          }
        }
        ```

        **Update the sink properties of a stream of type `sumo`**

        ```json
        {
          "sink": {
            "sumoSourceAddress": "string"
          }
        }
        ```

        Parameters
        ----------
        id : str
            The id of the log stream to get

        name : typing.Optional[str]
            log stream name

        status : typing.Optional[LogStreamStatusEnum]

        is_priority : typing.Optional[bool]
            True for priority log streams, false for non-priority

        filters : typing.Optional[typing.Sequence[LogStreamFilter]]
            Only logs events matching these filters will be delivered by the stream. If omitted or empty, all events will be delivered.

        pii_config : typing.Optional[LogStreamPiiConfig]

        sink : typing.Optional[LogStreamSinkPatch]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateLogStreamResponseContent]
            Log stream updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"log-streams/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "status": status,
                "isPriority": is_priority,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Sequence[LogStreamFilter], direction="write"
                ),
                "pii_config": convert_and_respect_annotation_metadata(
                    object_=pii_config, annotation=LogStreamPiiConfig, direction="write"
                ),
                "sink": convert_and_respect_annotation_metadata(
                    object_=sink, annotation=LogStreamSinkPatch, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateLogStreamResponseContent,
                    parse_obj_as(
                        type_=UpdateLogStreamResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .email_provider_credentials import EmailProviderCredentials
from .email_provider_settings import EmailProviderSettings


class UpdateEmailProviderResponseContent(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the email provider. Can be `mailgun`, `mandrill`, `sendgrid`, `resend`, `ses`, `sparkpost`, `smtp`, `azure_cs`, `ms365`, or `custom`.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the provider is enabled (true) or disabled (false).
    """

    default_from_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address to use as "from" when no other address specified.
    """

    credentials: typing.Optional[EmailProviderCredentials] = None
    settings: typing.Optional[EmailProviderSettings] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionCreatedObject7OptionsUseridAttributeEnum = typing.Union[
    typing.Literal["oid", "sub"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

SetsCustomTextsByLanguageRequestContent = typing.Dict[str, typing.Any]
"""
An object containing custom dictionaries for a group of screens.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawUsersClient, RawUsersClient


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def delete_by_email(self, id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a specified connection user by its email (you cannot delete all users from specific connection). Currently, only Database Connections are supported.

        Parameters
        ----------
        id : str
            The id of the connection (currently only database connections are supported)

        email : str
            The email of the user to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.users.delete_by_email(
            id="id",
            email="email",
        )
        """
        _response = self._raw_client.delete_by_email(id, email=email, request_options=request_options)
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def delete_by_email(
        self, id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a specified connection user by its email (you cannot delete all users from specific connection). Currently, only Database Connections are supported.

        Parameters
        ----------
        id : str
            The id of the connection (currently only database connections are supported)

        email : str
            The email of the user to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.users.delete_by_email(
                id="id",
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_by_email(id, email=email, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.delete_user_identity_response_content import DeleteUserIdentityResponseContent
from ...types.user_id import UserId
from ...types.user_identity import UserIdentity
from ...types.user_identity_provider_enum import UserIdentityProviderEnum
from .raw_client import AsyncRawIdentitiesClient, RawIdentitiesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class IdentitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawIdentitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawIdentitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawIdentitiesClient
        """
        return self._raw_client

    def link(
        self,
        id: str,
        *,
        provider: typing.Optional[UserIdentityProviderEnum] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        user_id: typing.Optional[UserId] = OMIT,
        link_with: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserIdentity]:
        """
        Link two user accounts together forming a primary and secondary relationship. On successful linking, the endpoint returns the new array of the primary account identities.

        Note: There are two ways of invoking the endpoint:

        - With the authenticated primary account's JWT in the Authorization header, which has the `update:current_user_identities` scope:

          ```http
          POST /api/v2/users/PRIMARY_ACCOUNT_USER_ID/identities
          Authorization: "Bearer PRIMARY_ACCOUNT_JWT"
          {
            "link_with": "SECONDARY_ACCOUNT_JWT"
          }
          ```

          In this case, only the `link_with` param is required in the body, which also contains the JWT obtained upon the secondary account's authentication.

        - With a token generated by the API V2 containing the `update:users` scope:

          ```http
          POST /api/v2/users/PRIMARY_ACCOUNT_USER_ID/identities
          Authorization: "Bearer YOUR_API_V2_TOKEN"
          {
            "provider": "SECONDARY_ACCOUNT_PROVIDER",
            "connection_id": "SECONDARY_ACCOUNT_CONNECTION_ID(OPTIONAL)",
            "user_id": "SECONDARY_ACCOUNT_USER_ID"
          }
          ```

          In this case you need to send `provider` and `user_id` in the body. Optionally you can also send the `connection_id` param which is suitable for identifying a particular database connection for the 'auth0' provider.

        Parameters
        ----------
        id : str
            ID of the primary user account to link a second user account to.

        provider : typing.Optional[UserIdentityProviderEnum]
            Identity provider of the secondary user account being linked.

        connection_id : typing.Optional[str]
            connection_id of the secondary user account being linked when more than one `auth0` database provider exists.

        user_id : typing.Optional[UserId]
            user_id of the secondary user account being linked.

        link_with : typing.Optional[str]
            JWT for the secondary account being linked. If sending this parameter, `provider`, `user_id`, and `connection_id` must not be sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserIdentity]
            Identity successfully added.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.identities.link(
            id="id",
        )
        """
        _response = self._raw_client.link(
            id,
            provider=provider,
            connection_id=connection_id,
            user_id=user_id,
            link_with=link_with,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self,
        id: str,
        provider: UserIdentityProviderEnum,
        user_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteUserIdentityResponseContent:
        """
        Unlink a specific secondary account from a target user. This action requires the ID of both the target user and the secondary account.

        Unlinking the secondary account removes it from the identities array of the target user and creates a new standalone profile for the secondary account. To learn more, review [Unlink User Accounts](https://auth0.com/docs/manage-users/user-accounts/user-account-linking/unlink-user-accounts).

        Parameters
        ----------
        id : str
            ID of the primary user account.

        provider : UserIdentityProviderEnum
            Identity provider name of the secondary linked account (e.g. `google-oauth2`).

        user_id : str
            ID of the secondary linked account (e.g. `123456789081523216417` part after the `|` in `google-oauth2|123456789081523216417`).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserIdentityResponseContent
            User identity successfully unlinked.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.identities.delete(
            id="id",
            provider="ad",
            user_id="user_id",
        )
        """
        _response = self._raw_client.delete(id, provider, user_id, request_options=request_options)
        return _response.data


class AsyncIdentitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawIdentitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawIdentitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawIdentitiesClient
        """
        return self._raw_client

    async def link(
        self,
        id: str,
        *,
        provider: typing.Optional[UserIdentityProviderEnum] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        user_id: typing.Optional[UserId] = OMIT,
        link_with: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserIdentity]:
        """
        Link two user accounts together forming a primary and secondary relationship. On successful linking, the endpoint returns the new array of the primary account identities.

        Note: There are two ways of invoking the endpoint:

        - With the authenticated primary account's JWT in the Authorization header, which has the `update:current_user_identities` scope:

          ```http
          POST /api/v2/users/PRIMARY_ACCOUNT_USER_ID/identities
          Authorization: "Bearer PRIMARY_ACCOUNT_JWT"
          {
            "link_with": "SECONDARY_ACCOUNT_JWT"
          }
          ```

          In this case, only the `link_with` param is required in the body, which also contains the JWT obtained upon the secondary account's authentication.

        - With a token generated by the API V2 containing the `update:users` scope:

          ```http
          POST /api/v2/users/PRIMARY_ACCOUNT_USER_ID/identities
          Authorization: "Bearer YOUR_API_V2_TOKEN"
          {
            "provider": "SECONDARY_ACCOUNT_PROVIDER",
            "connection_id": "SECONDARY_ACCOUNT_CONNECTION_ID(OPTIONAL)",
            "user_id": "SECONDARY_ACCOUNT_USER_ID"
          }
          ```

          In this case you need to send `provider` and `user_id` in the body. Optionally you can also send the `connection_id` param which is suitable for identifying a particular database connection for the 'auth0' provider.

        Parameters
        ----------
        id : str
            ID of the primary user account to link a second user account to.

        provider : typing.Optional[UserIdentityProviderEnum]
            Identity provider of the secondary user account being linked.

        connection_id : typing.Optional[str]
            connection_id of the secondary user account being linked when more than one `auth0` database provider exists.

        user_id : typing.Optional[UserId]
            user_id of the secondary user account being linked.

        link_with : typing.Optional[str]
            JWT for the secondary account being linked. If sending this parameter, `provider`, `user_id`, and `connection_id` must not be sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserIdentity]
            Identity successfully added.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.identities.link(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.link(
            id,
            provider=provider,
            connection_id=connection_id,
            user_id=user_id,
            link_with=link_with,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self,
        id: str,
        provider: UserIdentityProviderEnum,
        user_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteUserIdentityResponseContent:
        """
        Unlink a specific secondary account from a target user. This action requires the ID of both the target user and the secondary account.

        Unlinking the secondary account removes it from the identities array of the target user and creates a new standalone profile for the secondary account. To learn more, review [Unlink User Accounts](https://auth0.com/docs/manage-users/user-accounts/user-account-linking/unlink-user-accounts).

        Parameters
        ----------
        id : str
            ID of the primary user account.

        provider : UserIdentityProviderEnum
            Identity provider name of the secondary linked account (e.g. `google-oauth2`).

        user_id : str
            ID of the secondary linked account (e.g. `123456789081523216417` part after the `|` in `google-oauth2|123456789081523216417`).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserIdentityResponseContent
            User identity successfully unlinked.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.identities.delete(
                id="id",
                provider="ad",
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, provider, user_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionCertsAd = typing.List[str]
"""
Array of X.509 certificates in PEM format used for validating SAML signatures from the AD identity provider (managed by the AD Connector agent).
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_auth_params_email import ConnectionAuthParamsEmail
from .connection_brute_force_protection import ConnectionBruteForceProtection
from .connection_disable_signup import ConnectionDisableSignup
from .connection_email_email import ConnectionEmailEmail
from .connection_options_common import ConnectionOptionsCommon
from .connection_totp_email import ConnectionTotpEmail


class ConnectionOptionsEmail(ConnectionOptionsCommon):
    """
    Options for the 'email' connection
    """

    auth_params: typing_extensions.Annotated[
        typing.Optional[ConnectionAuthParamsEmail],
        FieldMetadata(alias="authParams"),
        pydantic.Field(alias="authParams"),
    ] = None
    brute_force_protection: ConnectionBruteForceProtection
    disable_signup: typing.Optional[ConnectionDisableSignup] = None
    email: ConnectionEmailEmail
    name: str = pydantic.Field()
    """
    Connection name
    """

    totp: typing.Optional[ConnectionTotpEmail] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_stripe_create_portal_session_action import FlowActionStripeCreatePortalSessionAction
from .flow_action_stripe_create_portal_session_params import FlowActionStripeCreatePortalSessionParams
from .flow_action_stripe_create_portal_session_type import FlowActionStripeCreatePortalSessionType


class FlowActionStripeCreatePortalSession(UniversalBaseModel):
    id: str
    alias: typing.Optional[str] = None
    type: FlowActionStripeCreatePortalSessionType
    action: FlowActionStripeCreatePortalSessionAction
    allow_failure: typing.Optional[bool] = None
    mask_output: typing.Optional[bool] = None
    params: FlowActionStripeCreatePortalSessionParams

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_error import ActionError


class ActionExecutionResult(UniversalBaseModel):
    """
    Captures the results of a single action being executed.
    """

    action_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the action that was executed.
    """

    error: typing.Optional[ActionError] = None
    started_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the action was started.
    """

    ended_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the action finished executing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_organizations_members_roles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.members.roles.list_.0"
    client = get_client(test_id)
    client.organizations.members.roles.list(
        id="id",
        user_id="user_id",
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/organizations/id/members/user_id/roles",
        {"page": "1", "per_page": "1", "include_totals": "true"},
        1,
    )


def test_organizations_members_roles_assign() -> None:
    """Test assign endpoint with WireMock"""
    test_id = "organizations.members.roles.assign.0"
    client = get_client(test_id)
    client.organizations.members.roles.assign(
        id="id",
        user_id="user_id",
        roles=["roles"],
    )
    verify_request_count(test_id, "POST", "/organizations/id/members/user_id/roles", None, 1)


def test_organizations_members_roles_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "organizations.members.roles.delete.0"
    client = get_client(test_id)
    client.organizations.members.roles.delete(
        id="id",
        user_id="user_id",
        roles=["roles"],
    )
    verify_request_count(test_id, "DELETE", "/organizations/id/members/user_id/roles", None, 1)
